/* xfarbe.c:  X-Interface to FARB-E-2D
 * ----------------------------------------------------
 * Copyright (C) A.Preusser, Fritz-Haber-Inst. der MPG, 1993
 *                                                            */

/****** Includes   *****/
#include <stdio.h>
#include <X11/Intrinsic.h>	
#include <X11/StringDefs.h>
#include <X11/Shell.h>		
#include <X11/Xaw/Simple.h>        /* Athena Widgets */
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>        
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "resources.h"

/****** Function declarations *****/
void init_data();
void set_colors();
void Draw_image();
void Redisplay();
void Menu();
void Menu_done();
void Help();
void Write_post();
void Batch_xf();
void Quit();
void Compz();
void Labelc();
void Labela();
void Readct();
void Sequct();
void Resetct();
void Prof_start();
void Prof_move();
void Prof_end();
void Redraw();
void Version_ev();
void Help_ev();
void Appl_call();
void Add_symbols();
float itox();
float itoy();

/******  pointers for plotting ******/
void (*gplp)(), (*gfap)(), (*gsfaip)(), (*gtextp)(),
     (*gwatchp)(),(*gslwscp)();
void  gpl_ps(),  gfa_ps(),  gsfai_ps(),  gtext_ps(),
      gwatch_ps(), gslwsc_ps();

extern float xlen,ylen,xmarg,ymarg; /* defined in farbe2d.c */
int start_of_symb_colors; /* will be passed to mm.c */

/****** Symbolic constants *****/
#define XDINA4 29.7 /* size of DINA4 paper in x in cm */
#define YDINA4 21.  /* size of DINA4 paper in y in cm */
#define NXYMAX 513  /* (max No. of values in x and y) +1 */
#define NCELLS 500  /* No. of cells pointed to interactively*/
#define NP_PROF 1000 /* max.no. of points in profile */
#define abs(x) ((x) >= 0 ? (x) : -(x))

static int c__1 = 1;
static int c__m1 = -1;
static int c__5 = 5;
static int c__6 = 6;
static int c__4 = 4;
static int c__0 = 0;
static int c__2 = 2;

/****** Global data *****/
typedef struct {
	         float	      *z,c[NCOL];
                           /*  z= 2d-array, c= contour levels */
                 int           nx,ny,icol[NCOL+1],nc;
                 float         xscale, yscale, xyratio,
                               xscale1, yscale1;
                 float         xlen_read, ylen_read, x0_read, y0_read;
                 float         x[NXYMAX],y[NXYMAX];
                 float         xaxmin,xaxmax,yaxmin,yaxmax;
                 GC            gc;
		 Pixmap        pix;
                 int           lx,ly,lc,nat_num[NCOL+2];
                 char          clab [NCOL] [11]; 
		}
                 image_data, *image_data_ptr;

typedef struct {
	         float	       *x,*y,*height;
                 int           *i,*c,n;
		}
                 symbol_data, *symbol_data_ptr;
                /* n  = number of symbols 
                   i  = symbol types
                   c  = symbol colors     */

typedef struct {
                 int           nc[NCELLS], isc[NCELLS];
                 int           i;
                 float         x[NCELLS], y[NCELLS];
               }
                 single_cell_data, *s_c_data_ptr;
                /* nc = number of contours for single cell 
                   isc= index for first contour
                   i  = counter 
                   x,y= coordinates in cm clicked on  */


	static image_data data; /*data other than resources*/
        static image_data_ptr data_ptr;
        static symbol_data symb; /* data for symbols */
        static single_cell_data cell; /* interactive annotation*/
        res_data  reso;  /*data coming from resources*/

   static XtResource resources[] = {
        { XtNwidth, XtCWidth, XtRDimension, sizeof(Dimension),
          XtOffset(res_data_ptr,width), XtRString,     "707"},
        { XtNheight, XtCHeight, XtRDimension, sizeof(Dimension),
          XtOffset(res_data_ptr,height), XtRString,    "500"},
          /* seitenverhaeltnis: 1.414 (DINA4) 707:500 */

        { "ncol", "Ncol", XtRInt, sizeof(int),
          XtOffset(res_data_ptr,ncol), XtRString,        "16"},

        { "mode", "Mode", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,mode), XtRString,      "2"  },
        { "symbol_mode", "Mode", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,symb_mode), XtRString, "2"  },
        { "cell_mode", "Mode", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cell_mode), XtRString, "-1" },

        { "linewidth1", "Linewidth", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,lwidth[0]), XtRString,  "20"},
        { "linewidth2", "Linewidth", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,lwidth[1]), XtRString,  "40"},
        { "linewidth3", "Linewidth", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,lwidth[2]), XtRString,  "60"},

        { "char_height1", "CharHeight", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cheight[0]), XtRString,  "25"},
        { "char_height2", "CharHeight", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cheight[1]), XtRString,  "60"},
        { "char_height3", "CharHeight", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cheight[2]), XtRString,  "80"},

        { "file", XtCFile, XtRString, sizeof(String),
          XtOffset(res_data_ptr,filnam), XtRString,
                                              "xfarbe.dat"},
        { "psfile", XtCFile, XtRString, sizeof(String),
          XtOffset(res_data_ptr,psfil), XtRString,
                                              "xfarbe.ps"},
        { "palfile", XtCFile, XtRString, sizeof(String),
          XtOffset(res_data_ptr,palfil), XtRString,
                                              "xfarbe.pal"},
        { "symfile", XtCFile, XtRString, sizeof(String),
          XtOffset(res_data_ptr,symfil), XtRString,
                                              "nofile"},
        { "profile", XtCFile, XtRString, sizeof(String),
          XtOffset(res_data_ptr,prof_fil), XtRString,
                                              "xfarbe.prof"},

        { "plot_title", "Plot_title", XtRString, sizeof(String),
          XtOffset(res_data_ptr,plot_title), XtRString,
          "xfarbe  -  Fill Area with Bicubics  -  A. PREUSSER "},

	{ XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[0]), XtRString,      "white"},
	{ XtNbackground, XtCBackground, XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL]), XtRString,      "black"},

        { "col1", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[1]),XtRString,
                                            "#FFFFE8"},
        { "col2", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[2]),XtRString,
                                                "#E7FAFF"},
        { "col3", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[3]), XtRString,
                                            "#FFECBE"},
        { "col4", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[4]), XtRString,
                                                "#C0F2FF"},
        { "col5", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[5]), XtRString,
                                            "#FFCDB1"},
        { "col6", "Col", XtRPixel, sizeof(Pixel),
          XtOffset(res_data_ptr,col[6]),XtRString,
                                                "#8CE6FF"},
        { "col7", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[7]), XtRString,
                                            "#FFAA97"},
        { "col8", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[8]), XtRString,
                                                "#5DD2FF"},
        { "col9", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[9]),XtRString,
                                            "#FF8750"},
        { "col10", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[10]),XtRString,
                                                "#43AAFF"},
        { "col11", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[11]), XtRString,
                                            "#FF5A08"},
        { "col12", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[12]), XtRString,
                                                "#2878FF"},
        { "col13", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[13]), XtRString,
                                            "#DC2700"},
        { "col14", "Col", XtRPixel, sizeof(Pixel),
          XtOffset(res_data_ptr,col[14]),XtRString,
                                                "#0047D2"},
        { "col15", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[15]), XtRString,
                                            "#880000"},
        { "col16", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[16]), XtRString,
                                                "#000074"},
        { "col17", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[17]),XtRString,
                                                "#F3FFF2"},
        { "col18", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[18]),XtRString,
                                                "#FFEBF4"},
        { "col19", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[19]), XtRString,
                                                "#BEFFEB"},
        { "col20", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[20]), XtRString,
                                                "#EFD2FF"},
        { "col21", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[21]), XtRString,
                                                "#82FFD8"},
        { "col22", "Col", XtRPixel, sizeof(Pixel),
          XtOffset(res_data_ptr,col[22]),XtRString,
                                                "#C8C4EA"},
        { "col23", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[23]), XtRString,
                                                "#32FFD5"},
        { "col24", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[24]), XtRString,
                                                "#AFAAC8"},
        { "col25", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[25]),XtRString,
                                                "#1ED7D7"},
        { "col26", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[26]),XtRString,
                                                "#A579C3"},
        { "col27", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[27]), XtRString,
                                                "#1CA5A5"},
        { "col28", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[28]), XtRString,
                                                "#A03CB4"},
        { "col29", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[29]), XtRString,
                                            "#007664"},
        { "col30", "Col", XtRPixel, sizeof(Pixel),
          XtOffset(res_data_ptr,col[30]),XtRString,
                                                "#8000A0"},
        { "col31", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[31]), XtRString,
                                            "#003400"},
        { "col32", "Col", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[32]), XtRString,
                                                "#0F0011"},
/* col[33]= reso.bg (col[NDEFCOL]= Background color) */
        { "fg_c_labels", "Foregr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+1]), XtRString,
                                            "#000000"},
        { "bg_c_labels", "Backgr", XtRPixel, sizeof(Pixel),
          XtOffset(res_data_ptr,col[NDEFCOL+2]),XtRString,
                                                "#FFFFFF"},
        { "fg_max_labels", "Foregr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+3]), XtRString,
                                            "#000000"},
        { "bg_max_labels", "Backgr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+4]), XtRString,
                                                "#FF3322"},
        { "fg_min_labels", "Foregr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+5]), XtRString,
                                            "#000000"},
        { "bg_min_labels", "Backgr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+6]), XtRString,
                                                "#1FFF1F"},
        { "fg_sad_labels", "Foregr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+7]), XtRString,
                                            "#000000"},
        { "bg_sad_labels", "Backgr", XtRPixel, sizeof(Pixel),
	  XtOffset(res_data_ptr,col[NDEFCOL+8]), XtRString,
                                                "#777777"},

        { "font_1","ApplFont",XtRFont, sizeof(Font),
          XtOffset(res_data_ptr,font1), XtRString, 
                          "-adobe-courier-*-r-*-*-*-80*"},
        { "font_2","ApplFont",XtRFont, sizeof(Font),
          XtOffset(res_data_ptr,font2), XtRString, 
                          "-adobe-courier-bold-r-*-*-*-120*"},
        { "font_3","ApplFont",XtRFont, sizeof(Font),
          XtOffset(res_data_ptr,font3), XtRString, 
                          "-adobe-helvetica-bold-r-*-*-*-140*"},
        { "autolev", "Autolev", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,autolev), XtRString,   "on"},
        { "autopal", "Autopal", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,autopal), XtRString,   "on"},
        { "autoxgrid", "Autogrid", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,autoxgr), XtRString,   "on"},
        { "autoygrid", "Autogrid", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,autoygr), XtRString,   "on"},
        { "xyratio", "Xyratio", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,ratio), XtRString,    "off"},
        { "posta4", "Post", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,posta4), XtRString,     "off"},
        { "posta4p", "Post", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,posta4p), XtRString,     "off"},
        { "posta3", "Post", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,posta3), XtRString,     "off"},
        { "legend", "Post", XtRBoolean, sizeof(Boolean),
	  XtOffset(res_data_ptr,legend), XtRString,     "on"},
        { "rescol", "Rescol", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,rescol), XtRString,    "32"},

        {"x-annotation","AxisAnnotation",XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,xann), XtRString,   "off"},
        { "y-annotation","AxisAnnotation", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,yann), XtRString,   "off"},

        { "c-annotation","ContourAnnotation", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,cann), XtRString,   "off"},
        { "x-annotation-incr", "AxisAnnotation-increment", XtRInt,
          sizeof(int),
	  XtOffset(res_data_ptr,xannincr), XtRString,    "10"},
        { "y-annotation-incr", "AxisAnnotation-increment", XtRInt,
          sizeof(int),
	  XtOffset(res_data_ptr,yannincr), XtRString,    "5"},
        { "c-annotation-incr", "ContourAnnotation-increment",
          XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cannincr), XtRString,    "1"},
        { "c-annotation-start", "ContourAnnotation-start",
          XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cannstart), XtRString,    "1"},
        { "x-c-annotation-incr", "XYContourAnnotation-increment",
          XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,xcannincr), XtRString,    "2"},
        { "y-c-annotation-incr", "XYContourAnnotation-increment",
          XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,ycannincr), XtRString,    "0"},
        { "c-annotation-in-rect", "Max-Annotations",
          XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,cannmax), XtRString,    "1"},
        { "c-annotation-min-pts", "Min-annotation",
          XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,min_p_anno), XtRString,    "6"},

        { "x-annotation-format", "AxisAnnotation-format",
          XtRString,
          sizeof(String), XtOffset(res_data_ptr,xannform),
          XtRString,                                  " %.2f "},
        { "y-annotation-format", "AxisAnnotation-format",
          XtRString,
          sizeof(String), XtOffset(res_data_ptr,yannform),
          XtRString,                                 " %.2f "},

        { "c-annotation-format", "ContourAnnotation-format",
          XtRString,
          sizeof(String), XtOffset(res_data_ptr,cannform),
          XtRString,                                 " %.2f "},
        { "legend_format", "ContourAnnotation-format", XtRString,
          sizeof(String), XtOffset(res_data_ptr,l_form),
          XtRString,                                "default"},
        { "legend_mode", "Mode", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,leg_mode), XtRString, "-999"  },

	{ "tick-direction", "Tick-direction", XtRInt,
	  sizeof(int),
	  XtOffset(res_data_ptr,tickdir), XtRString,    "1"},
	{ "tick-length", "Tick-length", XtRInt,
	  sizeof(int),
	  XtOffset(res_data_ptr,ticklen), XtRString,    "10"},
	{ "tick-length-long", "Tick-length", XtRInt,
	  sizeof(int),
	  XtOffset(res_data_ptr,ticklenl), XtRString,   "15"},


        { "x-margin", "Margin", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,xmarg), XtRString,    "25"},
        { "y-margin", "Margin", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,ymarg), XtRString,    "20"},
        { "x-length", "Length", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,xlen), XtRString,    "160"},
        { "y-length", "Length", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,ylen), XtRString,    "160"},
        { "x-scaling","Scaling", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,xscal), XtRString,   "off"},
        { "y-scaling","Scaling", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,yscal), XtRString,   "off"},

        { "profile_points", "Points", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,np_prof), XtRString,    "100"},

        { "min-heights", "msm-height", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,min_height), XtRString,    "0"},
        { "min-annotation", "msm-annotation", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,min_anno), XtRString,    "off"},
        { "min-annotation-format", "msm-annotation-format",
          XtRString,
          sizeof(String), XtOffset(res_data_ptr,min_form),
          XtRString,                                 " %.3g "},

        { "sad-heights", "msm-height", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,sad_height), XtRString,    "0"},
        { "sad-annotation", "msm-annotation", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,sad_anno), XtRString,    "off"},
        { "sad-annotation-format", "msm-annotation-format",
          XtRString,
          sizeof(String), XtOffset(res_data_ptr,sad_form),
          XtRString,                                 " %.3g "},

        { "max-heights", "msm-height", XtRInt, sizeof(int),
	  XtOffset(res_data_ptr,max_height), XtRString,    "0"},
        { "max-annotation", "msm-annotation", XtRBoolean,
          sizeof(Boolean),
	  XtOffset(res_data_ptr,max_anno), XtRString,    "off"},
        { "max-annotation-format", "msm-annotation-format",
          XtRString,
          sizeof(String), XtOffset(res_data_ptr,max_form),
          XtRString,                                 " %.3g "},
        };

	static Widget canv, topLevel,
                      pshell,menu,
                      label1,
                      menudone,posta3,posta4,posta4p,batch,
                      readct,sequct,resetct,redraw,
                      labelcell, labelcont, help;
        XColor xcol[NCOL+2];
        Colormap cm;
        Boolean sequential_colors;
        static char xftitle[40]= "xfarbe ";
        static int forced_annotation;
        static int xcannincr, ycannincr;


/****** Main program *****/
        main(argc, argv)
        int argc;
        char **argv;
{
	Arg wargs[2];

/****** Hard coded Translations ******/
	static char hardTranslations[]= "#augment \n\
		        <Expose>:           expose() \n\
	        	<Configure>:        resize()\n\
		       	<Btn2Down>:         popup()\n\
                        <Btn3Down>:         compz()\n\
                        <Key>q:             quit()\n\
                        <Key>s, <Btn1Down>: labelc()\n\
                        <Key>a, <Btn1Down>: labela()\n\
                        <Key>p, <Btn1Down>: prof_start()\n\
		      <Key>p, <Btn1Motion>: prof_move()\n\
			<Key>p, <Btn1Up>:   prof_end()\n\
                        <Key>v:             version()\n\
                        <Key>h:             help()\n";

/****** Actions Table *****/
	static XtActionsRec actions[]= {
			 {"expose",     Redisplay },
			 {"resize",     Draw_image},
			 {"popup",      Menu      },
                         {"compz",      Compz     },
                         {"labelc",     Labelc    },
                         {"labela",     Labela    },
                         {"prof_start", Prof_start},
			 {"prof_move",  Prof_move },
			 {"prof_end",   Prof_end  },
                         {"version",    Version_ev},
                         {"help",       Help_ev   },
			 {"quit",       Quit      }, };
	XtTranslations newtrans;

/****** Initialize Toolkit *****/
        topLevel = XtInitialize(
        "xfarbe",               /* Application name */
	"XFarbe",               /* Application class */
        NULL, 0,                /* command line option list */
        &argc, argv);           /* command line args 
        NULL,                  
        NULL,0);                   terminate varargs list */

/****** Get application resources ****/
	XtGetApplicationResources (topLevel,&reso,resources,
	      			  XtNumber(resources),NULL,0);

/****** use application resources width and height -
            (defaults are hard-coded)                        */
	XtSetArg(wargs[0], XtNwidth, reso.width);
	XtSetArg(wargs[1], XtNheight, reso.height);
        XtSetValues(topLevel,wargs,2);

/****** Create drawing widget *****/
	canv = XtCreateManagedWidget(
		"xfarbe",  /* widget name for appl-resources */
		simpleWidgetClass, /* widget class from aw.h */
		topLevel,          /* parent widget */
		NULL,0);           /* terminate varargs list */

/****** Apply Translations ****/
	newtrans= XtParseTranslationTable(hardTranslations);
	XtAugmentTranslations (canv,newtrans); 

/****** Add actions ******/
	XtAddActions (actions,XtNumber(actions));

/****** Create transient Shell as popup shell ****/
	pshell = XtCreatePopupShell(
		"xfarbeMenue",
		transientShellWidgetClass,
		topLevel,
		NULL,0);

/****** Create Box as parent for the commands *****/
        menu = XtCreateManagedWidget(
               "box",
                boxWidgetClass,
                pshell,
                NULL,0);

/****** Create Command Buttons ********/
/*        label1 = XtCreateManagedWidget(
                 "Use button 3 for probing",
               labelWidgetClass,menu,NULL,0); */
        help   = XtCreateManagedWidget(
                 "help",
               commandWidgetClass,menu,NULL,0);
        posta4 = XtCreateManagedWidget(
                 "posta4",
               commandWidgetClass,menu,NULL,0);
        posta4p= XtCreateManagedWidget(
                 "posta4p",
               commandWidgetClass,menu,NULL,0);
        posta3 = XtCreateManagedWidget
                ("posta3",
               commandWidgetClass,menu,NULL,0);
        batch  = XtCreateManagedWidget(
                 "batch",
               commandWidgetClass,menu,NULL,0);
        readct  = XtCreateManagedWidget(
                 "readct",
               commandWidgetClass,menu,NULL,0);
        sequct  = XtCreateManagedWidget(
                 "sequct",
               commandWidgetClass,menu,NULL,0);
        resetct = XtCreateManagedWidget(
                 "resetct",
               commandWidgetClass,menu,NULL,0);
        redraw  = XtCreateManagedWidget(
                 "redraw",
               commandWidgetClass,menu,NULL,0);
/*        labelcell= XtCreateManagedWidget(
                 "labelcell",
               commandWidgetClass,menu,NULL,0);
        labelcont= XtCreateManagedWidget(
                 "labelcont",
               commandWidgetClass,menu,NULL,0);  */
        menudone = XtCreateManagedWidget(
                 "menudone",
               commandWidgetClass,menu,NULL,0);
        
/****** callbacks for Commands *******/ 
	XtAddCallback(help, XtNcallback, Help, NULL);
	XtAddCallback(posta4, XtNcallback, Write_post,4);
	XtAddCallback(posta4p, XtNcallback, Write_post,45);
 	XtAddCallback(posta3, XtNcallback, Write_post,3);
	XtAddCallback(batch, XtNcallback, Batch_xf,4);
	XtAddCallback(menudone, XtNcallback, Menu_done, NULL); 
        XtAddCallback(readct, XtNcallback, Readct, NULL);
        XtAddCallback(sequct, XtNcallback, Sequct, NULL);
 	XtAddCallback(resetct, XtNcallback, Resetct,NULL);  
        XtAddCallback(redraw, XtNcallback, Redraw, NULL);
/*        XtAddCallback(labelcell, XtNcallback, Labcell,NULL);  
        XtAddCallback(labelcont, XtNcallback, Labcont, NULL); */

/****** Create windows for widgets and map them. *****/
        /* do not map, if only postscript file is requested */

        if (reso.posta4 != 0 || reso.posta3 != 0 ||
            reso.posta4p !=0)                 {
           XtSetMappedWhenManaged(topLevel,0);}

	XtRealizeWidget(topLevel); 

/****** initialize and call plot programm *****/
        reso.filnam= (argc==2) ? argv[1] : reso.filnam;
	init_data(canv, reso.filnam);

        if (reso.autopal==0){ Readct(canv,NULL,NULL); 
                              Sequct(canv,NULL,NULL); }  

        if (reso.ncol>NCOL || reso.ncol<2) {
           printf ("ncol= %d illegal, setting to %d\n", 
                    reso.ncol, NDEFCOL);
           reso.ncol= NDEFCOL;
           reso.autolev= TRUE;}

        data.xyratio= 0.; if(reso.ratio!=0) data.xyratio= 1.;

        if (reso.posta4 != 0) {
           Write_post(canv, 4, NULL);
           Quit(canv,NULL,NULL,NULL);}
        else if 
           (reso.posta3 != 0) {
           Write_post(canv, 3, NULL);
           Quit(canv,NULL,NULL,NULL);}
        else if 
           (reso.posta4p != 0) {
           Write_post(canv, 45, NULL);
           Quit(canv,NULL,NULL,NULL);}
        else

        Appl_call();

/* for further calls, use returned, computed values for data.c */
        data.nc= reso.ncol -1;
        data.lc= 1;

/****** Loop for events. *****/ 
	XtMainLoop();
}
/*****************************************************/
 void Appl_call()
{ static int i, c__1 =1, cell_nc, cell_n;
  static float z[1], zx[1], zy[1];

/*** All cells ***/
        forced_annotation = 0;
	farbe_(data.z, &data.nx, &data.nx, &data.ny, &reso.mode,
               &reso.ncol, reso.plot_title, &data.xyratio,
               data.c, data.lc,
               data.x, data.lx,
               data.y, data.ly,
               reso.xmarg*0.1, reso.ymarg*0.1,
               reso.xlen*0.1, reso.ylen*0.1,
               xcannincr, ycannincr);

/*** frame is plotted after area filling contours ***/
 
/*** Interactive cells ***/
 cell_n= cell.i; /*save cell counter */
 cell.i= 0;
 for (i=1; i<=cell_n ;i++) {
        forced_annotation= 1;
        cell_nc= abs(cell.nc[i-1]); 
        /* when labelling single contours : mode= -1 */
        itplbv_(cell.nc[i-1] != -1 ? &reso.cell_mode : &c__m1,
                &data.nx, &data.ny, data.x, data.y,
                data.z, &c__1, /* ev. of 1 dummy point */
                &data.c[cell.isc[i-1]],
                &data.nat_num[cell.isc[i-1]],
                &cell_nc,
                &cell.x[i-1], &cell.y[i-1],  /*out:*/ z, zx, zy);
     /* clear fill area buffer ***/
        if (reso.cell_mode%2 == 0) frbfcl_(&data.nat_num[1]);
        cell.i++;
 }
 cell.i= cell_n; /* restore cell counter */


/*** Symbols ***/
        Add_symbols(symb.n);

printf ("\n");

}
/*****************************************************/

 void gfa_(n,x,y,
          jpolr, npolr, nride, jfrst, jtost)
	int *n;
	float *x,*y;
        int *jpolr,*npolr,*nride,*jfrst,*jtost;
/* the second line of parameters will not be used for X */
{
	register int i;
        typedef struct { short x,y;} Xpoint;
	static Xpoint points[201];
        extern int xtoi(), ytoi();

/* Parameter adjustment for f2c ******commented ******
--x;
--y; */

/* copy to Points structure */
	for (i=1;i<=*n;i++)
	{
	 points[i].x= xtoi(x[i]);
	 points[i].y= ytoi(y[i]);
	}

/* Fill area, draw lines for filling border with same color */
/*      Pixmap always */
        XFillPolygon(XtDisplay(canv), data.pix, data.gc,
              &points[1],*n,
	      Nonconvex,CoordModeOrigin);
/*        XDrawLines(XtDisplay(canv), data.pix, data.gc,
              &points[1],*n,
	      CoordModeOrigin); */

/*      Window, if realized */   
        if(XtIsRealized(canv)){   
        XFillPolygon(XtDisplay(canv), XtWindow(canv), data.gc,
              &points[1],*n,
	      Nonconvex,CoordModeOrigin);   
/*        XDrawLines(XtDisplay(canv), XtWindow(canv), data.gc,
              &points[1],*n,
	      CoordModeOrigin); */   }

}
/* ======================================================== */
void gsfai_(i)
	int *i;
{
	XSetForeground(XtDisplay(canv),data.gc,reso.col[*i]);
}
/* ======================================================== */

/* ARGSUSED */
void gpl_(n,x,y)
	int *n;
	float *x,*y;


{	register int i;
        typedef struct { short x,y;} Xpoint;
	static Xpoint points[201];

/* Parameter adjustment for f2c */
--x;
--y;

/* copy to Points structure */
	for (i=1;i<=*n;i++)
	{
	 points[i].x= xtoi(x[i]);
	 points[i].y= ytoi(y[i]);
	}
/*      set foreground */
        XSetForeground(XtDisplay(canv),data.gc,reso.fg);
/*      Pixmap always */
        XDrawLines(XtDisplay(canv), data.pix, data.gc,
              &points[1],*n,
	      CoordModeOrigin);
/*      Window, if realized */   
        if(XtIsRealized(canv)){ 
        XDrawLines(XtDisplay(canv), XtWindow(canv), data.gc,
              &points[1],*n,
	      CoordModeOrigin);   } 
}
/* ======================================================== */

/* ARGSUSED */
void gtext_(x, y, h, text, w, centerx, centery, color,
            back_color, n)
	float *x, *y;
	char text[];
	float w, centerx, centery;
	int *n, h, color, back_color;

{
 unsigned int ix,iy;
 int height, height_plus;
 Font font;
    static int ascent, descent, width, width_plus;
    static XFontStruct *pxf;

/*     TEXT PLOTTING */
/*     PARAMETERS ARE IDENTICAL WITH CALCOMP-ROUTINE */
/*                  SYMBOL */

/*     X,Y        COORDINATES OF LOWER LEFT CORNER OF TEXT */
/*     H          1,2,3 for three different fonts */
/*     TEXT       TEXT */
/*     centerx    0., 0.5, 1. for left justified,
                              centered, right justified */
/*     centery    0., 0.5, 1. for bottom reference,
                                  centered, top reference */
/*     color      index into reso.col(= pixel values) for foreground */
/*     back_color index into reso.col                 for background */
/*     N          NUMBER OF CHARACTERS IN TEXT */

    if (h==1) font= reso.font1;
    if (h==2) font= reso.font2;
    if (h==3) font= reso.font3;

    XSetFont(XtDisplay(canv),data.gc,font);
    pxf= XQueryFont(XtDisplay(canv),font);

    height= (pxf->ascent);

	ix= xtoi(*x);
	iy= ytoi(*y);

        iy= iy + (int)((height+1)*centery );

        width= XTextWidth(pxf,text,*n);
        width_plus= width + 2;

        ix= ix - (int) ((width+1)*centerx );
        height_plus= height + 2;

/* clear background */
	XSetForeground(XtDisplay(canv),data.gc,reso.col[back_color]);
        XFillRectangle(XtDisplay(canv),data.pix,data.gc,
	    ix,iy-height,width_plus,height_plus);
/* draw string */
	XSetForeground(XtDisplay(canv),data.gc,reso.col[color]);
	XDrawString (XtDisplay(canv),data.pix,data.gc,
		ix,iy, text, *n);
 
        if(XtIsRealized(canv)){
/* clear background */
	XSetForeground(XtDisplay(canv),data.gc,reso.col[back_color]);
        XFillRectangle(XtDisplay(canv),XtWindow(canv),data.gc,
	    ix,iy-height,width_plus,height_plus);
/* draw string */
	XSetForeground(XtDisplay(canv),data.gc,reso.col[color]);
	XDrawString (XtDisplay(canv),XtWindow(canv),data.gc,
		ix,iy, text, *n);} 
 
}
/* ======================================================== */
/* ARGSUSED */
void gwatch_(it)
      int it;
{
printf (".");
}
/* ======================================================== */
/* ARGSUSED */
void gslwsc_(iw)
      int iw;
{
}
/* ======================================================== */
/* ARGSUSED */
void init_data(w, filnam)
	Widget w;
        char  *filnam;
{
        FILE *fp, *fopen();
	int nxy,i,i1,j,k,ic, items;
        int *zp3,*spi,*spc;
	float *zp1,*zp2,*spx,*spy,*sph;
        char line[81], form[10], text1[11];

/* Check number of colors from resources */
        if (reso.ncol>NCOL) reso.ncol= NCOL;

/* Foreground and Background Pixels*/
         reso.fg= reso.col[0];
	 reso.bg= reso.col[NDEFCOL];

/* scaling factors from cm to pixel */
	data.xscale= reso.width/XDINA4;
	data.yscale= reso.height/YDINA4;
        data.xscale1= 1./data.xscale;
        data.yscale1= 1./data.yscale;

/* create graphics context */
	data.gc= XCreateGC(XtDisplay(w),
			DefaultRootWindow(XtDisplay(w)),
			0,NULL);

/* create pixmap */
	data.pix= XCreatePixmap(XtDisplay(w),
 		  DefaultRootWindow(XtDisplay(w)),
                  reso.width,reso.height,
	   	  DefaultDepthOfScreen(XtScreen(w)));

/* initialize pixmap */
	XSetForeground(XtDisplay(w),data.gc,reso.bg);
        XFillRectangle(XtDisplay(w),data.pix,data.gc,
			0,0,reso.width,reso.height);
/* open FILE filnam */
        if ((fp= fopen(filnam, "r")) == NULL) {
           fprintf (stderr, "Unable to open %s\n", filnam);
           exit(1); }

/* Add name of file to title for window manager */
        XStoreName (XtDisplay(topLevel),XtWindow(topLevel), 
                    strcat(xftitle,filnam));

/* Initialize plot pointers to point to X-routines */
        gfap= gfa_;
        gplp= gpl_;
        gsfaip= gsfai_;
        gtextp= gtext_;
        gwatchp= gwatch_;
        gslwscp=  gslwsc_;

/* read first line of file */
        printf ("Reading from file %s:\n",filnam);
        fgets (line, 81, fp);

/* try to read dimensions of array from line */
	if (sscanf(line,"%d %d",&data.nx,&data.ny)<2)

          /* if unsuccessful, read title and dimensions */
          {
            strcpy (reso.plot_title,line);
            fprintf (stderr,"First line: %s\n",reso.plot_title);
            fscanf(fp,"%d %d",&data.nx,&data.ny);
          }
	fprintf (stderr,"Dimensions of array: %d * %d\n",
                data.nx,data.ny);

/* stop, if dimensions are incorrect */
   if (data.nx<2 || data.ny<2) exit(0);

/* get storage for array and read array elements */
	nxy= data.nx*data.ny;
	if ((data.z= (float *) calloc(nxy,sizeof(float)))==NULL) 
	 fprintf (stderr,
         "Unable to get enough storage for %dx%d array\n",
	  data.nx,data.ny);
	else {
         zp1= data.z; 
	 for (k=0; k<nxy; k++)  {
            fscanf(fp,"%f", data.z++);     }
             }
	data.z= zp1;

/* set default values for color indices */
        for (k=0; k<=NCOL; k++) data.icol[k]= k; 

/* optionally, read contour levels and color indices */
        data.lc= 0;   /* default: automatic */
        if (reso.autolev == 0) {

   /* read no. of levels */
        if (fscanf(fp," %d", &reso.ncol)!=EOF) {
          fprintf(stderr,
          " Reading levels, because\n\
            xfarbe.autolev: off\n\
            No. of levels: %d reading from %s \n",
            reso.ncol,filnam);
          /* reso.ncol will be incremented by one later */
 
   /* read color indices to resources colors,
          value of contour levels,
          and optionally strings for annotation  */

          zp3= &data.icol[1]; zp2= data.c;
          fgets (line, 81, fp); /* finish current line */

          for (k=0; k<reso.ncol; k++) {
            fgets (line, 81, fp);
            items= sscanf(line,"%d %f %s ",
                                zp3++, zp2++,data.clab[k] );

            if (items < 2 || items > 3){
               fprintf (stderr,
               "Error in reading contour level %d\n",k+1);
         printf("icol= %d, c= %f, string= %s no. of items= %d\n",
                data.icol[k+1], data.c[k], data.clab[k], items);
                exit (3); 
            }
              
            /*if no string for annotation, encode contour level*/
            if (items == 2)  {strcpy(data.clab[k],"");} 

          }

  /* read one more color */    
            if (fscanf(fp,"%d", zp3)!=1)
               fprintf (stderr,"Error: Supply one more color\n");
             ++reso.ncol;          /* ncols= no of levs+1 */
             if (reso.ncol>NCOL) reso.ncol= NCOL; 
             data.lc= 1;     /* automatic off */
                                            }
           else
           reso.autolev= TRUE;  /* automatic on */
        }

/* optionally, read x- and y-coordinates for grid in cm */
      { int ncoord= data.nx;
        if (reso.mode==3) ncoord= data.nx + 1;    
             zp2= &data.x[0];
             data.lx= 0;  /* default: automatic */
             data.ly= 0;
        if (reso.autoxgr == 0) {
             for (k=0; k<ncoord; k++) { 
               if (fscanf(fp,"%f", zp2++)==EOF) 
                 reso.autoxgr= TRUE;     }
             if (reso.autoxgr == 0){
               data.lx= 1;
               fprintf(stderr,
                 "x-coordinates for grid read successfully\n");
	       if (reso.xscal != 0){  /* scale x-coordinates */
                  data.xlen_read= data.x[ncoord-1]-data.x[0];
                  data.x0_read=   data.x[0];
                  for (k=0; k<ncoord; k++) {
                     data.x[k]= reso.xmarg/10. +
                     (reso.xlen/10./data.xlen_read)*(data.x[k]-data.x0_read);}
                 }
               }    
        }
        if (reso.autoygr == 0) {
             ncoord= data.ny;
             if (reso.mode==3) ncoord= data.ny + 1;
             zp2= &data.y[0]; 
             for (k=0; k<ncoord; k++) { 
               if (fscanf(fp,"%f", zp2++)==EOF) 
                 reso.autoygr= TRUE;     }
             if (reso.autoygr== 0) { 
               data.ly= 1;
               fprintf(stderr,
                 "y-coordinates for grid read successfully\n");
	       if (reso.yscal !=0){  /* scale y-coordinates */
                  data.ylen_read= data.y[ncoord-1]-data.y[0];
                  data.y0_read=   data.y[0];
                  for (k=0; k<ncoord; k++) {
                     data.y[k]= reso.ymarg/10. + 
                     (reso.ylen/10./data.ylen_read)*(data.y[k]-data.y0_read);}
               }
             }
        }
      }
/* optionally, read xaxmin, xaxmax, yaxmin, yaxmax */
        if (reso.xann != 0) {
           i= fscanf(fp,"%f %f", &data.xaxmin,&data.xaxmax);
              if (i==EOF || i<2)  reso.xann= FALSE;
              if (i==0 || i==1) fprintf(stderr,
              "ERROR reading x-min and x-max for axis labels\n");
              
            if (reso.xann!= 0)
               fprintf(stderr,
                 "x-min and x-max value for axis: %f %f\n",
                 data.xaxmin,data.xaxmax);
        }

        if (reso.yann != 0) {
           i= fscanf(fp,"%f %f", &data.yaxmin,&data.yaxmax);
              if (i==EOF || i<2)  reso.yann= FALSE;
              if (i==0 || i==1) fprintf(stderr,
              "ERROR reading y-min and y-max for axis labels\n");

           if (reso.yann!= 0)
                 fprintf(stderr,
                 "y-min and y-max value for axis: %f %f\n",
                 data.yaxmin,data.yaxmax);
         }


/**** try to read symfile ***/
       /* open FILE symfil */
        if (strcmp(reso.symfil,"nofile") != 0) {
          if ((fp= fopen(reso.symfil, "r")) == NULL) {
           fprintf (stderr, "Unable to open %s\n", reso.symfil);
          }
          else {

           /* count lines */
           symb.n= 0;
           while ((i=getc(fp)) != EOF)
                if (i == '\n') ++symb.n;
           rewind(fp);

        symb.n= symb.n -2 ; /* two comment lines */
        fgets (line, 81, fp);
        fgets (line, 81, fp);           

       /* get storage  */	
	if ((symb.x= (float *)calloc(symb.n,sizeof(float)))==NULL
          ||(symb.y= (float *)calloc(symb.n,sizeof(float)))==NULL
          ||(symb.i= (int *)calloc(symb.n,sizeof(int)))==NULL
          ||(symb.c= (int *)calloc(symb.n,sizeof(int)))==NULL
    ||(symb.height= (float*)calloc(symb.n,sizeof(float)))==NULL) 
	 fprintf (stderr,
         "Unable to get enough storage for %d symbols\n",
	  symb.n);
	else {
         spi= symb.i; spc= symb.c; spx= symb.x; spy= symb.y;
         sph= symb.height;

         j= 0;
	 for (k=0; k<symb.n; k++)  {
          i= fscanf(fp,"%d %f %d %f %f",
          symb.i++, symb.height++, symb.c++, symb.x++, symb.y++);
          if (i < 5) fprintf(stderr,
"Error in reading symfile: %s \n\
Could only read %d items in line %d\n", reso.symfil, i, k+3);
          else ++j;
         }
	symb.i= spi; symb.height= sph; 
        symb.c= spc; symb.x= spx; symb.y= spy;

/*        for (k=0; k<symb.n; k++)
           *(symb.c+k)= reso.col[k+1]; */

        printf("number of symbols read from %s : %d\n",
               reso.symfil, j);
       }            
      }
     }

/* Set defaults for x-y-c-annotation-increments */
        xcannincr= reso.xcannincr;
        if (xcannincr==0) xcannincr= 2;
        ycannincr= reso.ycannincr;
        if (ycannincr==0) ycannincr=
                         (data.ny/data.nx)*xcannincr;
        if (ycannincr==0) ycannincr= 1;

        if (reso.cann == 0) {xcannincr= 0; ycannincr= 0;}

/* set natural numbers */
   for (k=0; k<=NCOL+1; k++) data.nat_num[k]= k;

/* initialize counter for interactive cells */
   cell.i = 0;

set_colors(w);
}
/* ================================================*/
void set_colors(w)
        Widget w;
/* Set colors */
  {     
        float r,g,b,ra,ga,ba,ri,gi,bi,rv,gv,bv,rjm;
        int jmax,cells_av,i,j,k,ic,itemp;
        float rki= 0.125;
        static float red   [8][2] ={ 1. ,  1. ,    /* 1 */
                                     1. ,  1.,     /* 2 */
                                     1.,   1.,     /* 3 */
                                     1. ,  0.8,
                                     0.8,  0.4,
                                     0.4 , 0.,
                                     0.0,  0.,
                                     0.0 , 0.};    /* 8 */

        static float green [8][2] ={ 1.,  0.7,   /* 1 */
                                     0.7, 0.4,     /* 2 */
                                     0.4, 0.0,    /* 3 */
                                     .0,  0., 
                                     0. , 0.,
                                     0. , 0.,
                                     0.,  0.5,
                                     0.5 ,0.9};    /* 8 */

        static float blue  [8][2] ={ 0.9, 0.0 ,   /* 1 */
                                     0.0, 1.0,     /* 2 */
                                     1., 0.0,     /* 3 */
                                     0.0, 1.0,
                                     1. , 0.0,
                                     0.0 , 1.,
                                     1.0,  0.,
                                     0.0 , 1.};    /* 8 */
 /*     save already allocated resource colors  */
        start_of_symb_colors= reso.ncol + 1;
        if (reso.ncol<NDEFCOL+8)
            start_of_symb_colors= NDEFCOL +9;
        for (i=1; i<=NDEFCOL+8; i++) {
            reso.col[start_of_symb_colors-1+i]= reso.col[i];}

        cells_av= 1; /* color cells are available */
        jmax= reso.ncol/64 +1;
        rjm= 1./(jmax);
        ic= 1;
	for (i=0; i<=7 && cells_av !=0; i++){
           ra= red[i][0];
           ri= (red[i][1]-red[i][0])/(jmax+1);
           ga= green[i][0];
           gi= (green[i][1]-green[i][0])/(jmax+1);
           ba= blue[i][0];
           bi= (blue[i][1]-blue[i][0])/(jmax+1);
           rv=0.; gv=0.; bv=0.; 
           if (ri==0.){if (ra==0.) rv= rki;
                       if (ra==1.) rv= -rki;}
           else if (gi==0.){if (ga==0.) gv= rki;
                       if (ga==1.) gv= -rki;}
           else if (bi==0.){if (ba==0.) bv= rki;
                       if (ba==1.) bv= -rki;}
           for (j=0; j<=jmax  && cells_av !=0; j++){

              for (k=0; k<=7  && cells_av !=0; k++){
                 if (ic>reso.ncol) break;                 
                 r= ra + k*rv;
                 g= ga + k*gv;
                 b= ba + k*bv;
                 xcol[ic].red=   r*65535.;
                 xcol[ic].green= g*65535.;
                 xcol[ic].blue=  b*65535.;
                 if (ic>reso.rescol)
                 {
                 cells_av=  XAllocColor(XtDisplay(w),
                     DefaultColormap(XtDisplay(w),
                     DefaultScreen(XtDisplay(w))),
                     &xcol[ic]);
                 if (cells_av==0) {  
                     reso.ncol= ic-1;
                     fprintf(stderr,
                            "only %d colors can be allocated \n\
                             no. of levels set to %d \n",
                             ic-1,ic-2);
                     /* save symbol colors to new location */
                     itemp= start_of_symb_colors;
                     start_of_symb_colors= reso.ncol + 1;
                     if (reso.ncol<NDEFCOL+8)
                     start_of_symb_colors= NDEFCOL +9;
                     for (i=1; i<=NDEFCOL+8; i++) {
                      reso.col[start_of_symb_colors-1+i]=
                               reso.col[i-1+itemp];}
                 }
                 else 
                     reso.col[ic]= xcol[ic].pixel;
                                    } 
                 ic++;
                 }  /* end k */
              ra+= ri;
              ga+= gi;
              ba+= bi;
             }   /* end j */
           }  /* end i */

    sequential_colors= 0;   /*colors_cells are allocated */
}  /* set colors */
/* ======================================================== */

/* ARGSUSED */
void Draw_image(w,event,params,num_params)
	Widget w;
	XConfigureEvent *event;
	String *params;
	Cardinal *num_params;
{
/* set new scale, height and width */
        if (event != NULL)
        {
	data.xscale= event->width/(xmarg+xlen+8.);
	data.yscale= event->height/(ymarg+ylen+2.);
        data.xscale1= 1./data.xscale;
        data.yscale1= 1./data.yscale;
	reso.width= event->width;
	reso.height= event->height;
        }

/* create new pixmap */
        XFreePixmap(XtDisplay(w),data.pix);
	data.pix= XCreatePixmap(XtDisplay(w),
 		  DefaultRootWindow(XtDisplay(w)),
	          reso.width,reso.height,
	   	  DefaultDepthOfScreen(XtScreen(w)));

/* clear screen */
	XSetForeground(XtDisplay(w),data.gc,reso.bg);
        XFillRectangle(XtDisplay(w),data.pix,data.gc,
			0,0,reso.width,reso.height);
        if(XtIsRealized(w)){   
        XFillRectangle(XtDisplay(w),XtWindow(w),data.gc,
			0,0,reso.width,reso.height);}

/* Call to drawing program */
	Appl_call();

}
/* ======================================================== */

/* ARGSUSED */
void Redisplay(w,event,params,num_params)
	Widget w;
	XExposeEvent *event;
	String *params;
	Cardinal *num_params;
{
/* copy exposed area from pixmap */
	XCopyArea(XtDisplay(w), data.pix, XtWindow(w), data.gc,
	          event->x,event->y,
	 	  event->width, event->height,
		  event->x, event->y); 
}
/* ======================================================== */

/* ARGSUSED */
void Quit(w,event,params,num_params)
	Widget w;
	XButtonEvent *event;
	String *params;
	Cardinal *num_params;
{
   XFreePixmap(XtDisplay(w),data.pix);
   XFreeColormap(XtDisplay(w),cm);
   exit(1);}
/* ======================================================== */

void Compz(w,event,params,num_params)
	Widget w;
	XButtonEvent *event;
	String *params;
	Cardinal *num_params;
{ int itask= -100;
  static float x[2],y[2],z[2],zx[2],zy[2],xt,yt;
  char text[80];
  int istrl;
  int c__1 = 1;
  if(XtIsRealized(canv) && reso.mode != 3){
        x[0]= itox(event->x);
        y[0]= itoy(event->y);
/*printf("x0= %f y0= %f\n",x[0],y[0]); */
        itplbv_(&itask, &data.nx, &data.ny, data.x, data.y,
                data.z, &c__1,
                data.c,  data.nat_num, &data.nc,
                x, y,  /*out:*/    z, zx, zy);
        xt= 0.;
        yt= itoy(0); 
  sprintf (text,"x=%6.2f y=%6.2f z=%10.3g zx=%10.3g zy=%10.3g\0",
                 x[0],y[0],z[0],zx[0],zy[0]);
        XSetFunction(XtDisplay(w),data.gc,GXinvert);

        XDrawPoint(XtDisplay(w),XtWindow(w),data.gc,event->x,
                                                    event->y);
        XSetFunction(XtDisplay(w),data.gc,GXcopy);  
        istrl= strlen(text);
        gtext_(&xt, &yt, 2, text, 0., 0., 1., 0, /* fg */
               start_of_symb_colors+NDEFCOL-1, /* bg */
               &istrl);
  }
}
/* ======================================================== */
int usrpsc_(x, y, n, ncol, mode,
            irect,
            jpolr, npolr, iride, jfrst, jtost)

	float *x, *y;
	int *n, *ncol, *mode;
        int *jpolr,*npolr,*iride,*jfrst,*jtost, irect;
{
       int i, i1, istrl, ixmax, iymax, k;
       float xmax, ymax, xt, yt, w, s, dx, dy, smax ;
       double dx2y2, dx2y2min, sqrt(), atan2();
       char text[20], text1[20];
       static int ic_ann;
       static int oldrect=0;
       static int newrect=1;
/*     usrpsc_ is the plot interface

       mode= 1, LINE DRAWING (possibly with labels)
            -1, only labels, NO LINE 
             0, FILL AREA 

       x,y      COORDINATES FOR POLYGON 
       n        NUMBER OF POINTS FOR POLYGON 
       ncol     COLOUR FOR POLYGON OR CONTOUR LEVEL 
                NCOL IS always ICOL(I) 
       irect    number of rectangle
                =0, no annotation
                sign is for normal derivative
*/
    /* Parameter adjustments */
    --y;
    --x;

if (*mode == 2) goto L1000;  /*fill area, most frequently used*/

/*  POLYLINE  ************   
    calls to gplp must be with x[1], y[1], because
    index adjustment for f2c is in effect with gplp,
    not with gfap! */

    if (*mode == 1) { gplp(n, &x[1], &y[1]);}   /* 1 !! */
    
    /* ADD LABEL */
    if (abs(*mode) == 1) {
      if (abs(irect) != oldrect) 
                          {oldrect= abs(irect); ic_ann=0;} 
    
/* decision about annotation */

      if (reso.cann != 0 && irect != 0 && *n>=reso.min_p_anno &&
              reso.cannincr != 0 && *ncol >= reso.cannstart &&
              ic_ann < reso.cannmax &&
              (*ncol-reso.cannstart)%reso.cannincr ==0 ||
                                /* resource annotation */
          
          forced_annotation != 0 && cell.nc[cell.i] == -1
                                 && ic_ann < reso.cannmax ||
                                /* single annotation */

          forced_annotation != 0 && reso.cannincr == 0 || 

          forced_annotation != 0  && *ncol >= reso.cannstart &&
              reso.cannincr != 0 &&
              (*ncol-reso.cannstart)%reso.cannincr ==0 
              && ic_ann < reso.cannmax
                                /* all annotation */
          ) {

/* find point for annotation */

   /* normal annotation */                                                       
        if (ic_ann%2 ==1 )   {
           k= *n<7 ? k= *n-3 : 4; /* default 4 */
           k= k<2 ? k=2 : k ; /*minimum of k: 2*/
        }
        if (ic_ann%2 ==0 )      {
           k= *n<7 ? k= *n-1 : *n-3;  /*default n-3 */
        }
        xt= x[k];
        yt= y[k];
        if (*n==2) {
           xt= (x[2] + x[1])*0.5;
           yt= (y[2] + y[1])*0.5;
        }

   /* use nearest point clicked on */
       
        if (cell.nc[cell.i] == -1) { /* single annotation */
          dx= (cell.x[cell.i] - xt);
          dy= (cell.y[cell.i] - yt);
          dx2y2min= dx*dx + dy*dy;
          for (i=1; i<=*n; i++) {
             dx= (cell.x[cell.i] - x[i]);
             dy= (cell.y[cell.i] - y[i]);
             dx2y2= dx*dx + dy*dy;
             if (dx2y2 < dx2y2min) {k= i; dx2y2min= dx2y2;}
          }
          xt= x[k];
          yt= y[k];
        }

/*           w= atan2( (double) dy, (double) dx); */
/*printf ("k= %d, xt,yt= %f %f \n",k,xt,yt); */

/*       generate annotation string */
           if (strcmp(data.clab[*ncol-1],"") != 0){
               strcpy(text,data.clab[*ncol-1]);
           }
           else {               /* encode contour level */
              sprintf(text1,
                  reso.cannform, data.c[*ncol-1]);
              /* remove blanks */
              i1= 0; i=0;
              while (text1[i] != '\0'){
                 if (text1[i] != ' ') {
                     text[i1]= text1[i]; i1++;}
                 i++;
              } text[i1]= '\0';
           }
           /* annotate contour */             
           istrl= strlen(text);
           gtextp(&xt, &yt, 1, text,
                  0., 0.5 ,0.5,
                  start_of_symb_colors + NDEFCOL , /* foreground */
                  start_of_symb_colors + NDEFCOL + 1, /*backgr*/
                  &istrl);
           ++ic_ann;
      
      }  
       return 0;
    }

L1000:
/*  FILL AREA    ************/
/*     do not plot, if ncol==0   */
    if (*ncol == 0) {return 0;}

/*     set fill area index  */   
    gsfaip(&data.icol[*ncol]); 
/*    printf ("ncol= %d %d\n",*ncol,data.icol[*ncol]); */

/*     fill area                 */
    gfap(n, &x[0], &y[0],
         jpolr, npolr, iride, jfrst, jtost);

    if (*mode == 2)    gplp(n, &x[1], &y[1]);
    return 0;
} 

/* ======================================================== */

int xtoi(x)
float x;
{
 return (x*data.xscale );
}
/* ======================================================== */
int ytoi(y)
float y;
{
 return (reso.height - y*data.yscale ); 
}
/* ======================================================== */

float itox(i)
int i;
{
 return (i/data.xscale );
}
/* ======================================================== */
float itoy(i)
int i;
{
 return ((reso.height - i)/data.yscale ); 
}
/* ======================================================== */
/* ARGSUSED */
void Help_ev(w,event,params,num_params)
	Widget w;
	XButtonEvent *event;
	String *params;
	Cardinal *num_params;
{ Help(w,NULL,NULL);}
/* ======================================================== */
/* ARGSUSED */
void Version_ev(w,event,params,num_params)
	Widget w;
	XButtonEvent *event;
	String *params;
	Cardinal *num_params;
{ printf ("xfarbe Version 2.1, July 1994\n\
Copyright(C) A.Preusser Fritz-Haber-Institut der MPG, Berlin\n");
}
/* ======================================================== */
/* ARGSUSED */
void Menu(w,event,params,num_params)
/**** action routine for popping up menu ***/
	Widget w;
	XButtonEvent *event;
	String *params;
	Cardinal *num_params;
{	Arg wargs[2];
/*** place window at cursor position ****/
	XtSetArg(wargs[0], XtNx, event->x_root);
	XtSetArg(wargs[1], XtNy, event->y_root);
        XtSetValues(pshell,wargs,2);
/*** pop up menu ***/
        XtPopup(pshell,XtGrabNone);
        XRaiseWindow(XtDisplay(w),XtWindow(pshell));}
/* ======================================================== */
/*ARGSUSED*/
void Menu_done(w, client_data, call_data)
        Widget w;
        int *client_data;
        int *call_data;
  {XtPopdown(pshell);}
/* ======================================================== */

/*ARGSUSED*/
void Help(w, client_data, call_data)
        Widget w;
        int *client_data;
        int *call_data;
{
 printf ("The following are the most important rules to note:\n\
          - Use \"Mouse Button 3\" for data probing,\n\
          - Use \"Key a, Button 1\" to add labels in a cell,\n\
          - Use \"Key s, Button 1\" to label a contour,\n\
          - Use \"Key q\" to quit,\n\
          Further documentation will be found in the files \n\
                  xfarbe.doc  and  xfarbe.help,\n\
         ");
}
/* ======================================================== */
/*ARGSUSED*/
void Write_post(w, client_data, call_data)
        Widget w;
        int client_data;
        int *call_data;

{       int length_of_color_table;
        extern a3_ps(), a4_ps(), a4_port_ps();

/* switch plot pointers to post script */
        gfap= gfa_ps;
        gplp= gpl_ps;
        gsfaip= gsfai_ps;
        gtextp= gtext_ps;
        gwatchp= gwatch_ps;
        gslwscp= gslwsc_ps;

        printf("Computing \
for postscript file %s A%d\n",reso.psfil,client_data);

/* select a4, a3, or a4_portrait */
        if (client_data==3) a3_ps(reso.psfil);
        else if (client_data==45) a4_port_ps(reso.psfil);
        else a4_ps(reso.psfil);

        length_of_color_table= start_of_symb_colors-1+NDEFCOL+8;        
        prolog_ps(topLevel,reso.col,
                  length_of_color_table,
                  reso.lwidth, reso.cheight);

/* call to application */
	Appl_call();

/* pointers back to X */
        gfap= gfa_;
        gplp= gpl_;
        gsfaip= gsfai_;
        gtextp= gtext_;
        gwatchp= gwatch_;
        gslwscp= gslwsc_;

/* close postscript file */
        close_ps();

        printf("file %s closed\n",
        reso.psfil);
}
/*===============================================*/
/*ARGSUSED*/
void Batch_xf(w, client_data, call_data)
        Widget w;
        int client_data;
        int *call_data;

/* setup call for shell script, using same data file and
                                palette file             */
{
        char batchcall[60]; 
        strcpy (batchcall,"xfarbe.batch ");
        strcat (batchcall,reso.filnam);
        strcat (batchcall," ");
        strcat (batchcall,reso.palfil);
        strcat (batchcall,"&");
   printf("%s \n",batchcall);
   system (batchcall);
}
/*===============================================*/
/*ARGSUSED*/
void Readct(w, client_data, call_data)
        Widget w;
        int *client_data;
        int *call_data;

/*   READ color table (palette) */

{       FILE *fp, *fopen();
        char rmap[256],bmap[256],gmap[256];
        float r,g,b;
        int i,ic;

/****   Create Colormap ****/
        cm= XCreateColormap (XtDisplay(w),XtWindow(w),
                     DefaultVisual
                    (XtDisplay(w),DefaultScreen(XtDisplay(w))),
                     AllocAll);

        if (cm==DefaultColormap(XtDisplay(w),
                    DefaultScreen(XtDisplay(w))))
/*      StaticColor and StaticGray unsupported */
        printf ("Sorry, your color map cannot be changed\n");

        XSetWindowColormap(XtDisplay(w),XtWindow(topLevel),cm);
        printf ("WindowColormap set \n");

/*      open palette file */            
        if (NULL == (fp = fopen(reso.palfil,"r"))) {
        fprintf(stderr,
                "Error on palette file open %s \n",reso.palfil);
        exit(2);
        }

/*      read palette */
        fread(rmap,1,256,fp);
        fread(gmap,1,256,fp);
        fread(bmap,1,256,fp);

        for (ic=0;ic<=NCOL+1;ic++){
           xcol[ic].red=   rmap[ic]*256;
           xcol[ic].green= gmap[ic]*256;
           xcol[ic].blue=  bmap[ic]*256;

/* set pixel values in xcol.pixel used in XStore Color */
           xcol[ic].pixel= ic;

           if (ic==0) {xcol[ic].red=   0;
                       xcol[ic].green= 0;
                       xcol[ic].blue=  0;
                     }
           if (ic==1) {xcol[ic].red=   65535;
                       xcol[ic].green= 65535;
                       xcol[ic].blue=  65535;
                    }

                   xcol[ic].flags= DoRed | DoGreen | DoBlue;
                   XStoreColor(XtDisplay(w),
                     cm,
                     &xcol[ic]);
         }

 /* increase address space for colors, save symbol colors */
        start_of_symb_colors= 257;
        for (i=1; i<=NDEFCOL+8; i++) {
            reso.col[start_of_symb_colors-1+i]= reso.col[i];}

        printf (" \n\
Click onto 'Sequential Colors' for sequential usage of map \n\
and 'Redraw image'.\n\
Click into xfarbe window for Map to become effective!!\n");
/*         XInstallColormap(XtDisplay(topLevel),cm);  */

}
/*=======================================*/
/*ARGSUSED*/
void Sequct(w, client_data, call_data)
        Widget w;
        int *client_data;
        int *call_data;
{
        int ic;
        sequential_colors= 1;
/*****  set new colors reso.col  *****/
        for (ic=0;ic<=reso.ncol;ic++){
           reso.col[ic]= ic + 1;       }
/*****  the other indices are not used: ***/
        for (ic=reso.ncol+1;ic<=NCOL+1;ic++){
           reso.col[ic]= 0;}
/*****  the symbol colors pixel values are changed ****/
        for (ic=1;ic<=NDEFCOL+8;ic++){
           reso.col[start_of_symb_colors-1+ic]= ic;       }
        printf("Warning: \n \
Pixel values changed. What you see \
is  not  what you get.\n\
Redraw image and  Click into xfarbe window !!\n");
}
/*=======================================*/
/*ARGSUSED*/
void Resetct(w, client_data, call_data)
        Widget w;
        int *client_data;
        int *call_data;
{
        int nc, width, height;
/*      Set Default Colormap */
        XSetWindowColormap(XtDisplay(w),XtWindow(topLevel),
                           DefaultColormap(XtDisplay(w), 
                           DefaultScreen(XtDisplay(w)))); 
/*      reset reso.col  */
        if (sequential_colors==1)
            /* save changed resource values */
           {nc= reso.ncol;
            width= reso.width;
            height= reso.height;

           XtGetApplicationResources (topLevel,&reso,resources,
	      			  XtNumber(resources),NULL,0);

           /* reset changed resource values */

           reso.width= width;
           reso.height= height;

           /* allocate color cells again */
           set_colors(w);
           reso.ncol= nc;
 
           Redraw(w,NULL,NULL); }

        printf ("WindowColormap reset \n");
}
/*=======================================*/
/*ARGSUSED*/
void Redraw(w, client_data, call_data)
        Widget w;
        int *client_data;
        int *call_data;
{ Draw_image(canv, NULL, NULL, NULL);}
/*=======================================*/
/*ARGSUSED*/
void Labela(w, event, params, num_params)
        Widget w;
        XButtonEvent *event;
        String *params;
        Cardinal *num_params;
{ 
    static int c__1= 1;
    static float x[2], y[2], z[2], zx[2], zy[2];
    if(XtIsRealized(canv) &&
       reso.mode != 3     &&
       cell.i < NCELLS)     {
        cell.x[cell.i]= itox(event->x);
        cell.y[cell.i]= itoy(event->y);
        cell.nc[cell.i] = data.nc;
        cell.isc[cell.i]= 0;
        /* label single cell */

        forced_annotation= 1;
        itplbv_(&reso.cell_mode,
                &data.nx, &data.ny, data.x, data.y,
                data.z, &c__1, /* ev. of 1 dummy point */
                data.c,  data.nat_num, &data.nc,
                &cell.x[cell.i], &cell.y[cell.i],
                /*out:*/z, zx, zy);

        /* clear fill area buffer */
        if (reso.cell_mode%2 == 0) frbfcl_(&data.nat_num[1]);
        
       /* increment cell counter */
       cell.i++;
       if (cell.i > NCELLS-1) 
       fprintf(stderr,
       "No more than %d interactively drawn cells\n\
        increase parameter NCELLS\n\
        You are really working hard!!\n", NCELLS);
                  
    }
}
/*=======================================*/

/*ARGSUSED*/
void Labelc(w, event, params, num_params)
        Widget w;
        XButtonEvent *event;
        String *params;
        Cardinal *num_params;
{
    static int c__1= 1, itask= -100, i, ic;
    static float z[1], zx[1], zy[1];
    if(XtIsRealized(canv) &&
       reso.mode != 3 &&
       cell.i < NCELLS)     {
        cell.x[cell.i]= itox(event->x);
        cell.y[cell.i]= itoy(event->y);

/*** find contour level from horizontal position */
    itplbv_(&itask, &data.nx, &data.ny, data.x, data.y,
                data.z, &c__1,
                data.c,  data.nat_num, &data.nc,
                &cell.x[cell.i], &cell.y[cell.i],
                /*out:*/    z, zx, zy);

    /* find index of contour level from 0 to nc-1 */
    ic= 0;
    for (i=0; i<data.nc-1; i++) {
       if (z[0]>data.c[i] && z[0]<=data.c[i+1]) {ic= i; break;}
    }
    if (z[0] > data.c[0]) {
        if (data.c[ic+1]-z[0] < z[0]-data.c[ic]) ic= ic+1;
    }
    if (z[0] > data.c[data.nc-1]) ic= data.nc-1;

/*** label single cell, one line only */
        cell.nc[cell.i] = -1;
        cell.isc[cell.i]= ic;

        forced_annotation= 1;
        itplbv_(&c__1,
                &data.nx, &data.ny, data.x, data.y,
                data.z, &c__1, /* ev. of 1 dummy point */
                &data.c[ic],  &data.nat_num[ic], &c__1,
                &cell.x[cell.i], &cell.y[cell.i],
                /*out:*/z, zx, zy);

        /* clear fill area buffer */
        if (reso.cell_mode%2 == 0) frbfcl_(&data.nat_num[ic+1]);
        
       /* increment cell counter */
       cell.i++;
       if (cell.i > NCELLS-1) 
       fprintf(stderr,
       "No more than %d interactively drawn cells\n\
        increase parameter NCELLS\n\
        You are really working hard!!\n", NCELLS);        
    }
}
/*=======================================*/
 /* globals for Profiles */
 static float xprof1,yprof1,xprof2,yprof2;
 static int ixprof2, iyprof2, ixprof1, iyprof1;
 static int profile=0;

/*ARGSUSED*/
void Prof_start(w, event, params, num_params)
        Widget w;
        XButtonEvent *event;
        String *params;
        Cardinal *num_params;
{
    ixprof1= event->x;
    iyprof1= event->y;
    xprof1= itox(ixprof1);
    yprof1= itoy(iyprof1);
 
    printf ("First point of profile: %6.2f %6.2f\n",
              xprof1,yprof1);

    profile= 1;

/*    XSetForeground(XtDisplay(w),data.gc,
     BlackPixelOfScreen(DefaultScreenOfDisplay(XtDisplay(w)))); */
/* BlackPixel(XtDisplay(w),DefaultScreen(XtWindow(w))) );   */
}

/*ARGSUSED*/
void Prof_move(w, event, params, num_params)
        Widget w;
        XButtonEvent *event;
        String *params;
        Cardinal *num_params;
{
 if(XtIsRealized(canv) && profile>0){ 
  
   XSetFunction(XtDisplay(w),data.gc,GXinvert);

   if (profile > 1) {

   /*Undraw previous line*/
        XDrawLine(XtDisplay(canv), XtWindow(canv), data.gc,
             ixprof1, iyprof1, ixprof2, iyprof2); 
   }

   ixprof2= event->x;
   iyprof2= event->y;

   /*draw new line*/
        XDrawLine(XtDisplay(canv), XtWindow(canv), data.gc,
             ixprof1, iyprof1, ixprof2, iyprof2); 

   profile++;

   XSetFunction(XtDisplay(w),data.gc,GXcopy);  
 }
}

/*ARGSUSED*/
void Prof_end(w, event, params, num_params)
        Widget w;
        XButtonEvent *event;
        String *params;
        Cardinal *num_params;
{
 static float xprof[NP_PROF], yprof[NP_PROF];
 float z[NP_PROF],zx[NP_PROF],zy[NP_PROF];
 static FILE *fp;
 static int c100= -100, i;
 
 if (profile>0 && reso.mode != 3){
    ixprof2= event->x;
    iyprof2= event->y;
    xprof2= itox(ixprof2);
    yprof2= itoy(iyprof2);
    printf ("Secnd point of profile: %6.2f %6.2f\n",
              xprof2,yprof2);

/* generate array */
    for (i=0; i<reso.np_prof; i++) {    
       xprof[i]= xprof1 + i*(xprof2-xprof1)/(reso.np_prof-1);
       yprof[i]= yprof1 + i*(yprof2-yprof1)/(reso.np_prof-1);
    }
/* call interpolation */
    itplbv_(&c100,
                &data.nx, &data.ny, data.x, data.y,
                data.z, &reso.np_prof,
                data.c,  data.nat_num, &c__1,
                xprof, yprof,
                /*out:*/z, zx, zy);

/* write to file */
   if ((fp= fopen(reso.prof_fil, "w")) == NULL) {
       fprintf (stderr, "Unable to open %s\n", reso.prof_fil);
       }
   else {
     for (i=0; i<reso.np_prof; i++) {
      fprintf (fp,"%.2f %.2f %.5e\n", xprof[i], yprof[i], z[i]);
     }
     printf ("%d points profile written to %s\n",
              reso.np_prof, reso.prof_fil);
     fclose(fp);
   }

    profile= 0;
 }
}

/*=======================================*/
/*start of legena_ */
/*     legend and annotation for axes */

        int legena_(xmin, xmax, ymin, ymax, c, icol, nc,
                             form, mode, incr, xt, yt)
        float *xmin, *xmax, *ymin, *ymax, *c;
        int *icol, *nc;
        char *form;
        int *mode; int *incr;
        float *xt, *yt;
{
    /* System generated locals */
    int i__1;
    float r__1, r__2;

    /* Local variables */
    static int ncol, nx1, ny1, local_mode;
    static float ddxy;
    static char text[16];
    static int i, istrl;
    static float x[5], y[5], xx[5], yy[5];
    static float x0, y0, dx, dy, height, xlablen;
    static int istart;
    static float xt0, yt0;
    static float dmx, dmy;
    static char blank[3];
    static int ascent, descent, width;
    static XFontStruct *pxf;
/*     INPUT PARAMETERS */
/*     XMIN,XMAX */
/*     YMIN,YMAX   CORNERS OF FILLED AREA IN CM */
/*     C       CONTOUR LEVELS */
/*     ICOL    COLOURS FOR FILL AREA */
/*     NC      NUMBER OF CONTOUR LEVELS */
/*     FORM    FORMAT TO BE USED WHEN DISPLAYING VALUES OF C */
/*     MODE    0, FILL AREA ONLY */
/*             1, LINES ONLY */
/*             2, BOTH */
/*     INCR    INCREMENT FOR ANNOTATION OF CONTOUR LEVELS */

/*     NUMBERS ARE ALWAYS PLOTTED TO THE RIGHT SIDE OF THE FILLED */
/*     AREA, WHEN LOOKING FROM THE BOTTOM TO THE TOP */


    /* Parameter adjustments */
    --icol;
    --c;

if (reso.legend != 0) {
    /* Function Body */
    local_mode= *mode;
    if (reso.leg_mode != -999) local_mode= reso.leg_mode;
    ddxy = (float).3;
    gslwscp(0); /*set line width for contours */

/*     DIVIDE FILLED AREA INTO RECTANGLES */
    ncol = *nc + 1;
    x0 = *xmin;
    y0 = *ymin;

/*     DECIDE, IF INCREMENTATION IN X- OR Y-DIRECTION */
    dx = (float)0.;
    dmx = *xmax - *xmin;
    dmy = (float)0.;
    if ((r__1 = *xmax - *xmin, abs(r__1)) >
               (r__2 = *ymax - *ymin, abs(r__2)))
	     {
	dx = (r__1 = *xmax - *xmin, abs(r__1)) / ncol;
	dmy = *ymax - *ymin;
	dmx = (float)0.;
    }

    dy = (*ymax - *ymin) / ncol;
    if ((r__1 = *xmax - *xmin, abs(r__1)) >
        (r__2 = *ymax - *ymin, abs(r__2)))
	     {
	dy = (float)0.;
    }

    /* get length of ordinary label */
    XSetFont(XtDisplay(canv),data.gc,reso.font2);
    pxf= XQueryFont(XtDisplay(canv),reso.font2);
    /* take label for first level as example */
        sprintf
          (text,(strcmp(reso.l_form,"default")==0)?
                              form : reso.l_form
           ,c[0]); 
        istrl= strlen(text);
        xlablen= itox(XTextWidth(pxf,text,istrl));

    i__1 = ncol;
    for (i = 1; i <= i__1; ++i) {
	y[0] = y0;
	y[1] = y0;
	y[2] = y0 + dy + dmy;
	y[3] = y[2];
	x[0] = x0;
	x[1] = x0 + dx + dmx;
	x[2] = x[1];
	x[3] = x0;
        x[4] = x0;
        y[4] = y0;
/*       FILL AREA */
	if (local_mode != 1) {
/*	    usrplt_(x, y, &c__4, &icol[i], &c__0); */
	    usrpsc_(x, y, &c__4, &icol[i], &c__0,
                    0,   /* no annotation */
                    NULL,NULL,&c__0,NULL,NULL);
	}

	if (local_mode == 0 || local_mode == 3) {
	    goto L50;}
	istart = 1;

	if (dy == (float)0.) {
	    istart = 4;}

/*	usrplt_(&x[istart - 1],&y[istart - 1],&c__2, &icol[i],&c__1);*/
	usrpsc_(&x[istart - 1], &y[istart - 1],
                &c__2, &icol[i],&c__1,
                0,
                NULL,NULL,&c__0,NULL,NULL);
L50:

	if (i == ncol) {
	    goto L100;}

/*       SET START OF TEXT */
	xt0 = x[2] + ddxy;
	yt0 = y[2];
	if (dy == (float)0.) {
	    xt0 = x[1] - height * (float).5;
	    yt0 = y[1] - ddxy;
	}

/*       ENCODE VALUE OF CONTOUR LEVEL */
        sprintf
          (text,(strcmp(reso.l_form,"default")==0)?
                              form : reso.l_form
           ,c[i]); 

	if (*incr!=0 && i % *incr == 0) {
            istrl= strlen(text);
	    gtextp(&xt0, &yt0, 2, text, 0., 0.,0.5,
		start_of_symb_colors + NDEFCOL, /*forgr*/
              	start_of_symb_colors + NDEFCOL + 1,/*backgr*/     
	        &istrl);

/*  add label for annotated contours */

           if (strcmp(data.clab[i-1],"") != 0){
             strcpy(text,data.clab[i-1]);
             istrl= strlen(text);
             xt0= xt0 + xlablen + 2*ddxy;
             gtextp(&xt0, &yt0, 2, text, 0., 0. ,0.5,
		start_of_symb_colors + NDEFCOL, /*forgr*/
              	start_of_symb_colors + NDEFCOL + 1,/*backgr*/                    
		&istrl);
           }
	}

	y0 += dy;
	x0 += dx;
L100:
	;
    }
/*     PLOT BORDER FOR FILLED AREA around legend */
    gslwscp(1); /* select line width for border */
    x[0] = *xmin;
    x[1] = *xmax;
    x[2] = *xmax;
    x[3] = *xmin;
    x[4] = *xmin;
    y[0] = *ymin;
    y[1] = *ymin;
    y[2] = *ymax;
    y[3] = *ymax;
    y[4] = *ymin;
    gplp(&c__5, x, y); 
}

 nx1= data.nx;
 ny1= data.ny;
 if (reso.mode == 3) {
    nx1= nx1+1;
    ny1= ny1+1;
 }

/* x-axis annotation */
if (reso.xann != 0) {
    float xsc, xlab, ylab;
    xsc= (data.xaxmax-data.xaxmin)/(xt[nx1-1]-xt[0]);
    for (i=0; i<nx1; i++) {
        if (reso.xannincr != 0 && i%reso.xannincr == 0)    {
          sprintf
          (text,reso.xannform,data.xaxmin + (xt[i]-xt[0])*xsc);
          i__1= strlen(text);
           xlab= xt[i];
           ylab= yt[0] - 0.3;
           gtextp(&xlab, &ylab, 2, text, 0., 0.5, 1.0,
                  0,			          /* fg */
		  start_of_symb_colors+NDEFCOL-1, /* bg */
                  &i__1);}
    }
}

/* y-axis annotation */
if (reso.yann != 0) {
    float ysc, xlab, ylab;
    ysc= (data.yaxmax-data.yaxmin)/(yt[ny1-1]-yt[0]);
    for (i=0; i<ny1; i++) {
        if (reso.yannincr != 0 && i%reso.yannincr == 0)      {
           sprintf (text,reso.yannform,
                      data.yaxmin + (yt[i]-yt[0])*ysc);
          i__1= strlen(text);
           xlab= xt[0] - 0.3;
           ylab= yt[i];
           gtextp(&xlab, &ylab, 2, text, 0., 1.0, 0.5,
                  0,                             /* fg */
		start_of_symb_colors+NDEFCOL-1, /* bg */
		 &i__1);}
    }
}

/*     SET TEXT HEIGHT, PLOT HEADING */
    XSetFont(XtDisplay(canv),data.gc,reso.font3);
    pxf= XQueryFont(XtDisplay(canv),reso.font3);

    height= (pxf->ascent-1)*data.yscale1;
    r__1= xt[0];
    r__2= yt[ny1-1] + 1.0;
    i__1= strlen(reso.plot_title);
    gtextp(&r__1,&r__2, 3, reso.plot_title, 0., 0., 0.,
           0,                               /* fg */
	    start_of_symb_colors+NDEFCOL-1, /* bg */
           &i__1);

    gslwscp(0); /*set line width for contours */
  
} /* legena_ */

/************************************************/
int xf_frame (xt, yt)
float *xt, *yt;

/*     PLOT FRAME */
{float xx[5], yy[5];
 int nx1, ny1, i, i__1;
 nx1= data.nx;
 ny1= data.ny;
 if (reso.mode == 3) {
    nx1= nx1+1;
    ny1= ny1+1;
 }
    gslwscp(1);
    xx[0] = xt[0];
    yy[0] = yt[0];
    xx[1] = xt[nx1 - 1];
    yy[1] = yt[0];
    xx[2] = xt[nx1 - 1];
    yy[2] = yt[ny1 - 1];
    xx[3] = xt[0];
    yy[3] = yt[ny1 - 1];
    xx[4] = xx[0];
    yy[4] = yy[0];
    gplp(&c__5, xx, yy); 


/*     PLOT TICK MARKS */
    yy[0] = yt[0];
    i__1 = nx1;
    for (i = 1; i <= i__1; ++i) {
	xx[0] = xt[i - 1];
	xx[1] = xx[0];
        yy[1] = yt[0] - reso.tickdir*reso.ticklen*0.01;
        if (reso.xannincr != 0 && (i-1)%reso.xannincr == 0)
          yy[1]= yt[0] - reso.tickdir*reso.ticklenl*0.01;
	gplp(&c__2, xx, yy);
    }

    yy[0] = yt[ny1 - 1];
    i__1 = nx1;
    for (i = 1; i <= i__1; ++i) {
	xx[0] = xt[i - 1];
	xx[1] = xx[0];
        yy[1] = yt[ny1 - 1] + reso.tickdir*reso.ticklen*0.01;
        if (reso.xannincr != 0 && (i-1)%reso.xannincr == 0)
          yy[1]= yt[ny1-1] + reso.tickdir*reso.ticklenl*0.01;
	gplp(&c__2, xx, yy);
    }

    xx[0] = xt[0];
    i__1 = ny1;
    for (i = 1; i <= i__1; ++i) {
	yy[0] = yt[i - 1];
	yy[1] = yy[0];
        xx[1] = xt[0] - reso.tickdir*reso.ticklen*0.01;
        if (reso.yannincr != 0 && (i-1)%reso.yannincr == 0)
          xx[1]= xt[0] - reso.tickdir*reso.ticklenl*0.01;
	gplp(&c__2, xx, yy);
    }

    xx[0] = xt[nx1 - 1];
    i__1 = ny1;
    for (i = 1; i <= i__1; ++i) {
	yy[0] = yt[i - 1];
	yy[1] = yy[0];
        xx[1] = xt[nx1 - 1] + reso.tickdir*reso.ticklen*0.01;
        if (reso.yannincr != 0 && (i-1)%reso.yannincr == 0)
          xx[1]= xt[nx1-1] + reso.tickdir*reso.ticklenl*0.01;
	gplp(&c__2, xx, yy);
    }

}

/************************************************/
int gcirc (x, y, radius, color, mode)
float x, y;
float radius;
int color, mode;
{
    /* Builtin functions */
    double atan(), sin(), cos();

    static int i, j ;
    static int c__17 = 17;
    static int c__0 = 0;
    static float  pi8, xtemp[18], ytemp[18], pi, factor;

    double dtmp;

    dtmp= 1.;
    pi8 = atan(dtmp) / 2.;
    dtmp=0.;
		for (j = 1; j <= c__17; ++j) {

		    xtemp[j ] = x + radius * sin(dtmp);
		    ytemp[j ] = y + radius* cos(dtmp);
                    dtmp+= pi8;
        	}

    gsfaip(&color);
    gfap(&c__17, &xtemp[0], &ytemp[0],
                NULL,NULL,&c__0,NULL,NULL);
    if (mode == 1 || mode == 2) {
       gplp(&c__17, &xtemp[1], &ytemp[1]);} 


}
/************************************************/
int gdiam (x, y, height, color, mode)
float x, y;
float height;
int color, mode;
{
    static int c__5 = 5;
    static float  xtemp[6], ytemp[6], h2;

    h2= height;
    xtemp[1]= x + h2;
    xtemp[2]= x;
    xtemp[3]= x - h2;
    xtemp[4]= x;
    xtemp[5]= x + h2;

    ytemp[1]= y;
    ytemp[2]= y - h2;
    ytemp[3]= y;
    ytemp[4]= y + h2;
    ytemp[5]= y;

    gsfaip(&color);
    gfap(&c__5, &xtemp[0], &ytemp[0],
                NULL,NULL,&c__0,NULL,NULL);
    if (mode == 1 || mode == 2) {
       gplp(&c__5, &xtemp[1], &ytemp[1]);} 


}
/************************************************/
int gquad (x, y, height, color, mode)
float x, y;
float height;
int color, mode;
{
    static int c__5 = 5;
    static float  xtemp[6], ytemp[6], h2;

    h2= height;
    xtemp[1]= x + h2;
    xtemp[2]= x + h2;
    xtemp[3]= x - h2;
    xtemp[4]= x - h2;
    xtemp[5]= x + h2;

    ytemp[1]= y + h2;
    ytemp[2]= y - h2;
    ytemp[3]= y - h2;
    ytemp[4]= y + h2;
    ytemp[5]= y + h2;
    gsfaip(&color);
    gfap(&c__5, &xtemp[0], &ytemp[0],
                NULL,NULL,&c__0,NULL,NULL);
    if (mode == 1 || mode == 2) {
       gplp(&c__5, &xtemp[1], &ytemp[1]);} 

}
/************************************************/
void Add_symbols(ns)
int ns;

{int i,ip, mode;
 float xs, ys, h;

 for (i=0; i<ns; i++) {

    xs= *(symb.x + i);
        if (reso.autoxgr != 0) {  /* auto is on */
            xs= xmarg + xs*xlen; /* no x-values read */  
        }
        else if (reso.xscal != 0){  /* auto is off and xscal is on */
                xs= reso.xmarg/10. +
                    (reso.xlen/10./data.xlen_read)*(xs-data.x0_read);}
    ys= *(symb.y + i);
        if (reso.autoygr != 0) { /* auto is on */
            ys= ymarg + ys*ylen; /* no y-values read */      
        }
        else if (reso.yscal != 0){  /* auto is off and yscal is on */
                ys= reso.ymarg/10. +
                    (reso.ylen/10./data.ylen_read)*(ys-data.y0_read);}

    h= *(symb.height + i)*0.5;
/*printf("%f %f %f \n", xs,ys,h); */
    ip= start_of_symb_colors + *(symb.c + i) -1;
        /* ip= color of symbol i+1*/
/*printf("start_of..= %d ip= %d %d \n",
         start_of_symb_colors,ip,reso.col[ip]); */
    if (*(symb.i+i) == 1)
       gcirc (xs, ys, h, ip, reso.symb_mode);
    if (*(symb.i+i) == 2)
       gdiam (xs, ys, h, ip, reso.symb_mode);
    if (*(symb.i+i) == 3)
       gquad (xs, ys, h, ip, reso.symb_mode);
  }
}
